// @(#)root/tmva $Id: SVKernelFunction.cxx 29122 2009-06-22 06:51:30Z brun $    
// Author: Andrzej Zemla

/**********************************************************************************
 * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
 * Package: TMVA                                                                  *
 * Class  : SVKernelFunction                                                      *
 * Web    : http://tmva.sourceforge.net                                           *
 *                                                                                *
 * Description:                                                                   *
 *      Implementation                                                            *
 *                                                                                *
 * Authors (alphabetical):                                                        *
 *      Marcin Wolter  <Marcin.Wolter@cern.ch> - IFJ PAN, Krakow, Poland          *
 *      Andrzej Zemla  <azemla@cern.ch>        - IFJ PAN, Krakow, Poland          *
 *      (IFJ PAN: Henryk Niewodniczanski Inst. Nucl. Physics, Krakow, Poland)     *   
 *                                                                                *
 * Copyright (c) 2005:                                                            *
 *      CERN, Switzerland                                                         * 
 *      MPI-K Heidelberg, Germany                                                 * 
 *      PAN, Krakow, Poland                                                       *
 *                                                                                *
 * Redistribution and use in source and binary forms, with or without             *
 * modification, are permitted according to the terms listed in LICENSE           *
 * (http://tmva.sourceforge.net/LICENSE)                                          *
 **********************************************************************************/

#include "TMVA/SVKernelFunction.h"
#include "TMVA/SVEvent.h"
#include "TMath.h"
#include <vector>

//_______________________________________________________________________
TMVA::SVKernelFunction::SVKernelFunction()
   : fGamma(0.) 
{
   // constructor
}

//_______________________________________________________________________
TMVA::SVKernelFunction::SVKernelFunction( Float_t gamma )
   : fGamma(gamma)
{
   // constructor
}

//_______________________________________________________________________
TMVA::SVKernelFunction::~SVKernelFunction() 
{
   // destructor
}

//_______________________________________________________________________
Float_t TMVA::SVKernelFunction::Evaluate( SVEvent* ev1, SVEvent* ev2 )
{
   std::vector<Float_t> *v1 = ev1->GetDataVector();
   std::vector<Float_t> *v2 = ev2->GetDataVector();

   Float_t norm = 0;
   for (UInt_t i = 0; i < v1->size(); i++) norm += ((*v1)[i] -(*v2)[i]) *((*v1)[i] -(*v2)[i]) ;

   return TMath::Exp(-norm*fGamma);
}

